package il.ac.tau.cs.software1.my_music.ui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

public class GraphicalMyMusicViewer {

	private static final String APPLICATION_CREATION_DATE = "<date>";
	private static final String YOUR_NAME = "<your_name>";
	public static void main(String[] args) {
		showViewer();
	}

	public static void showViewer() {
		GraphicalMyMusicViewer viewer = new GraphicalMyMusicViewer();
		viewer.show();
	}

	private Shell shell; // the main application window
	private Label albumStatus; // the status label for albums
	private Tree albumsTree; // the tree in the albums tab
	private Label artistsStatus; // the artists status label
	private Tree artistsTree; // the tree in the artists tab

	private GraphicalMyMusicViewer() {
		// constructor is private, instances can only be created from this class
	}

	private void show() {
		createShell();
		runApplication();
	}

	/**
	 * Constructs the main window of the application
	 */
	private void createShell() {
		Display display = Display.getDefault();
		shell = new Shell(display);
		shell.setText(GUIUtils.MY_MUSIC);

		// window style
		Rectangle monitor_bounds = shell.getMonitor().getBounds();
		shell.setSize(new Point(monitor_bounds.width / 3,
				monitor_bounds.height / 2));
		shell.setLayout(new FillLayout());

		// On closing the window, if there are unsaved changes, the user may
		// save the changes before exiting
		shell.addDisposeListener(new DisposeListener() {

			@Override
			public void widgetDisposed(DisposeEvent e) {
				// TODO complete
			}
		});

		// create the different components within the main window
		createMenu();
		createTabs();
	}

	/**
	 * Creates the menu bar of the main window
	 */
	private void createMenu() {
		Menu menu = new Menu(shell, SWT.BAR);
		shell.setMenuBar(menu);

		createActionsMenu(menu);
		createAboutMenu(menu);
	}

	/**
	 * Constructs the "about" button in the menu bar
	 */
	private void createAboutMenu(Menu menu) {
		MenuItem aboutItem = new MenuItem(menu, SWT.PUSH);
		aboutItem.setText("About");
		aboutItem.addSelectionListener(new SelectionAdapter() {
			// TODO complete
		});
	}

	/**
	 * Opens a pop-up displaying the details about the program
	 */
	private void showAbout() {
		GUIUtils.showInfoDialog(shell, "About My Music",
				"My Music was created by " + YOUR_NAME + " on "
						+ APPLICATION_CREATION_DATE
						+ "\nAll rights reserved");
	}

	/**
	 * Creates the "Actions" menu in the menu bar
	 */
	private void createActionsMenu(Menu menu) {
		// create the "Actions" button and menu
		MenuItem actionsItem = new MenuItem(menu, SWT.CASCADE);
		actionsItem.setText("Actions");

		Menu actionsMenu = new Menu(shell, SWT.DROP_DOWN);
		actionsItem.setMenu(actionsMenu);

		// The option for creating a new user (and logging-in as this user)
		MenuItem newItem = new MenuItem(actionsMenu, SWT.PUSH);
		newItem.setText("New User...");
		newItem.addSelectionListener(new SelectionAdapter() {
			// TODO complete
		});

		// The option for logging-in by opening the data where some user's
		// albums were saved
		MenuItem openItem = new MenuItem(actionsMenu, SWT.PUSH);
		openItem.setText("Open User...");
		openItem.addSelectionListener(new SelectionAdapter() {
			// TODO complete
		});

		// The option for saving the user data into a file
		// Only enabled when there is a user
		MenuItem saveItem = new MenuItem(actionsMenu, SWT.PUSH);
		saveItem.setText("Save");
		saveItem.addSelectionListener(new SelectionAdapter() {
			// TODO complete
		});

		new MenuItem(actionsMenu, SWT.SEPARATOR);

		// The option to exit the application
		MenuItem exitItem = new MenuItem(actionsMenu, SWT.PUSH);
		exitItem.setText("Exit");
		exitItem.addSelectionListener(new SelectionAdapter() {
			// TODO complete
		});
	}

	/**
	 * Opens a log-in dialog which allows entering the details of a new user and
	 * logging-in as this user. If there are unsaved changes offers to save them
	 * first
	 */
	private void createNewUser() {
		// TODO complete
	}

	/**
	 * Loads all the user details (including the albums and artists) from the
	 * data file associated with this user If there are unsaved changes - offers
	 * to save them first.
	 * 
	 * @return true iff succeeded in loading the user data (no errors or
	 *         cancellations)
	 */
	private boolean openUser() {
		// TODO complete
	}

	/**
	 * Saves all the user data into the data file associated with the user.
	 * Checks if there are changes to save or not and updates the unsaved
	 * changes indication in the title
	 */
	private boolean doSave() {
		// TODO complete
	}

	/**
	 * Constructs the window title. "My Music" is the basic title
	 * "My Music - <user-name>" when there is a logged-in user "*" is added when
	 * there are unsaved changes
	 */
	private void updateWindowTitle() {
		// TODO complete
	}

	/**
	 * If there are unsaved changes, the user can choose to save them before the
	 * window is closed. (Hint: use org.eclipse.swt.widgets.Shell.dispose)
	 */
	private void exitApplication() {
		// TODO complete
	}

	/**
	 * Opens a confirm dialog, whether to save the changes or not. save the
	 * changes if the user presses yes
	 * 
	 * @param canCancel
	 *            - whether the "Cancel" option is available.
	 * @return false iff the operation was cancelled or the save failed.
	 */
	private boolean openSaveConfirm(boolean canCancel) {
		int style = SWT.ICON_QUESTION | SWT.YES | SWT.NO;
		if (canCancel) {
			style = style | SWT.CANCEL;
		}
		MessageBox messageBox = new MessageBox(shell, style);
		messageBox
				.setMessage("There are unsaved changes. Would you like to save them first?");
		int response = messageBox.open();
		switch (response) {
		case SWT.YES:
			return doSave();
		case SWT.CANCEL:
			return false;
		default:
			break;
		}
		return true;
	}

	/**
	 * Create the tabs in the main window - the albums tab and the artists tab
	 */
	private void createTabs() {
		TabFolder tabFolder = new TabFolder(shell, SWT.BORDER);

		createViewAlbumsTab(tabFolder);
		createViewArtistsTab(tabFolder);

		tabFolder.pack();
	}

	/**
	 * Creates the albums tab
	 * 
	 * @param tabFolder
	 */
	private void createViewAlbumsTab(TabFolder tabFolder) {
		// the tab itself
		TabItem albumsTab = new TabItem(tabFolder, SWT.NONE);
		albumsTab.setText("Albums");

		// the main container in the tab
		Composite albumsContent = new Composite(tabFolder, SWT.None);
		albumsContent.setLayout(new GridLayout(2, false));
		albumsTab.setControl(albumsContent);

		// the status line in the tab
		albumStatus = new Label(albumsContent, SWT.NONE);
		albumStatus.setLayoutData(new GridData(
				GridData.FILL_HORIZONTAL));

		// the "Add Album" button
		Button newAlbumButton = new Button(albumsContent, SWT.PUSH);
		newAlbumButton.setText("Add Album");
		newAlbumButton.addSelectionListener(new SelectionAdapter() {
			// TODO complete
		});

		// the tree with albums (initially empty)
		albumsTree = new Tree(albumsContent, SWT.SINGLE | SWT.BORDER);
		GridData treeLayoutData = new GridData(GridData.FILL_BOTH);
		treeLayoutData.horizontalSpan = 2;
		albumsTree.setLayoutData(treeLayoutData);
	}

	/**
	 * Opens a dialog where the user can enter all the details of an album
	 * (including artists and tracks) The new album will be added to the users
	 * albums and displayed in the UI.
	 */
	private void addAlbum() {
		// TODO complete
	}

	/**
	 * Creates the tab with artists
	 * 
	 * @param tabFolder
	 */
	private void createViewArtistsTab(TabFolder tabFolder) {
		// the tab itself
		TabItem albumsTab = new TabItem(tabFolder, SWT.NONE);
		albumsTab.setText("Artists");

		// The tab main container
		Composite artistsContent = new Composite(tabFolder, SWT.None);
		artistsContent.setLayout(new GridLayout());
		albumsTab.setControl(artistsContent);

		// The artists status line
		artistsStatus = new Label(artistsContent, SWT.NONE);
		artistsStatus.setLayoutData(new GridData(
				GridData.FILL_HORIZONTAL));

		// The tree with artists and their albums (initially empty)
		artistsTree = new Tree(artistsContent, SWT.SINGLE
				| SWT.BORDER);
		artistsTree.setLayoutData(new GridData(GridData.FILL_BOTH));
	}

	/**
	 * Opens the application main window, and runs the "event loop" in the
	 * standard SWT way.
	 */
	private void runApplication() {
		shell.open();
		Display display = shell.getDisplay();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch())
				display.sleep();
		}
		display.dispose();
	}
}
